@@ -19,6 +19,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import functools
 import re
 
 from tensorflow.contrib.framework.python.ops import add_arg_scope as contrib_add_arg_scope
@@ -30,14 +31,13 @@ from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import control_flow_ops
-from tensorflow.python.ops import init_ops
 from tensorflow.python.ops import variable_scope
-from tensorflow.python.ops import variables
 from tensorflow.python.ops import gen_state_ops
 from tensorflow.python.platform import tf_logging as logging
 from tensorflow.python.platform import resource_loader
 from tensorflow.python.training import saver as tf_saver
 from tensorflow.python.training import training_util
+from tensorflow.python.util.deprecation import deprecated
 
 
 __all__ = ['add_model_variable',
@@ -53,9 +53,11 @@ __all__ = ['add_model_variable',
            'get_or_create_global_step',
            'get_local_variables',
            'get_model_variables',
+           'get_trainable_variables',
            'get_unique_variable',
            'get_variables_by_name',
            'get_variables_by_suffix',
+           'get_variable_full_name',
            'get_variables_to_restore',
            'get_variables',
            'local_variable',
@@ -81,7 +83,7 @@ def zero_initializer(ref, use_locking=True, name="zero_initializer"):
       resource_loader.get_path_to_datafile("_variable_ops.so"))
   return gen_variable_ops.zero_initializer(ref, name=name)
 
-
+@deprecated(None, "Please switch to tf.train.assert_global_step")
 def assert_global_step(global_step_tensor):
   training_util.assert_global_step(global_step_tensor)
 
@@ -109,54 +111,40 @@ def assert_or_get_global_step(graph=None, global_step_tensor=None):
     assert_global_step(global_step_tensor)
   return global_step_tensor
 
-
+@deprecated(None, "Please switch to tf.train.get_global_step")
 def get_global_step(graph=None):
   return training_util.get_global_step(graph)
 
-
+@deprecated(None, "Please switch to tf.train.create_global_step")
 def create_global_step(graph=None):
   """Create global step tensor in graph.
 
+  This API is deprecated. Use core framework training version instead.
+
   Args:
-    graph: The graph in which to create the global step. If missing, use default
-        graph.
+    graph: The graph in which to create the global step tensor. If missing,
+      use default graph.
 
   Returns:
     Global step tensor.
 
   Raises:
-    ValueError: if global step key is already defined.
+    ValueError: if global step tensor is already defined.
   """
-  graph = ops.get_default_graph() if graph is None else graph
-  if get_global_step(graph) is not None:
-    raise ValueError('"global_step" already exists.')
-  # Create in proper graph and base name_scope.
-  with graph.as_default() as g, g.name_scope(None):
-    collections = [ops.GraphKeys.GLOBAL_VARIABLES, ops.GraphKeys.GLOBAL_STEP]
-    return variable(
-        ops.GraphKeys.GLOBAL_STEP,
-        shape=[],
-        dtype=dtypes.int64,
-        initializer=init_ops.zeros_initializer(),
-        trainable=False,
-        collections=collections)
-
+  return training_util.create_global_step(graph)
 
+@deprecated(None, "Please switch to tf.train.get_or_create_global_step")
 def get_or_create_global_step(graph=None):
-  """Returns and create (if necessary) the global step variable.
+  """Returns and create (if necessary) the global step tensor.
 
   Args:
-    graph: The graph in which to create the global step. If missing, use default
-        graph.
+    graph: The graph in which to create the global step tensor. If missing, use
+      default graph.
 
   Returns:
-    the tensor representing the global step variable.
+    The global step tensor.
   """
-  graph = ops.get_default_graph() if graph is None else graph
-  globalstep = get_global_step(graph)
-  if globalstep is None:
-    globalstep = create_global_step(graph)
-  return globalstep
+  return training_util.get_or_create_global_step(graph)
 
 
 def local_variable(initial_value, validate_shape=True, name=None):
@@ -169,7 +157,7 @@ def local_variable(initial_value, validate_shape=True, name=None):
   Returns:
     New variable.
   """
-  return variables.Variable(
+  return variable_scope.variable(
       initial_value, trainable=False,
       collections=[ops.GraphKeys.LOCAL_VARIABLES],
       validate_shape=validate_shape, name=name)
@@ -179,7 +167,7 @@ def local_variable(initial_value, validate_shape=True, name=None):
 def variable(name, shape=None, dtype=None, initializer=None,
              regularizer=None, trainable=True, collections=None,
              caching_device=None, device=None,
-             partitioner=None, custom_getter=None):
+             partitioner=None, custom_getter=None, use_resource=None):
   """Gets an existing variable with these parameters or creates a new one.
 
   Args:
@@ -204,17 +192,20 @@ def variable(name, shape=None, dtype=None, initializer=None,
       partitions for each axis (currently only one axis can be partitioned).
     custom_getter: Callable that allows overwriting the internal
       get_variable method and has to have the same signature.
+    use_resource: If `True` use a ResourceVariable instead of a Variable.
 
   Returns:
     The created or existing variable.
   """
-  collections = list(collections or [ops.GraphKeys.GLOBAL_VARIABLES])
+  collections = list(collections if collections is not None
+                     else [ops.GraphKeys.GLOBAL_VARIABLES])
 
   # Remove duplicates
   collections = set(collections)
   getter = variable_scope.get_variable
   if custom_getter is not None:
-    getter = custom_getter
+    getter = functools.partial(custom_getter,
+                               reuse=variable_scope.get_variable_scope().reuse)
   with ops.device(device or ''):
     return getter(name, shape=shape, dtype=dtype,
                   initializer=initializer,
@@ -222,14 +213,15 @@ def variable(name, shape=None, dtype=None, initializer=None,
                   trainable=trainable,
                   collections=collections,
                   caching_device=caching_device,
-                  partitioner=partitioner)
+                  partitioner=partitioner,
+                  use_resource=use_resource)
 
 
 @contrib_add_arg_scope
 def model_variable(name, shape=None, dtype=dtypes.float32, initializer=None,
                    regularizer=None, trainable=True, collections=None,
                    caching_device=None, device=None, partitioner=None,
-                   custom_getter=None):
+                   custom_getter=None, use_resource=None):
   """Gets an existing model variable with these parameters or creates a new one.
 
   Args:
@@ -255,6 +247,7 @@ def model_variable(name, shape=None, dtype=dtypes.float32, initializer=None,
       partitions for each axis (currently only one axis can be partitioned).
     custom_getter: Callable that allows overwriting the internal
       get_variable method and has to have the same signature.
+    use_resource: If `True` use a ResourceVariable instead of a Variable.
 
   Returns:
     The created or existing variable.
@@ -265,7 +258,8 @@ def model_variable(name, shape=None, dtype=dtypes.float32, initializer=None,
                  initializer=initializer, regularizer=regularizer,
                  trainable=trainable, collections=collections,
                  caching_device=caching_device, device=device,
-                 partitioner=partitioner, custom_getter=custom_getter)
+                 partitioner=partitioner, custom_getter=custom_getter,
+                 use_resource=use_resource)
   return var
 
 
@@ -328,6 +322,19 @@ def get_local_variables(scope=None, suffix=None):
   return get_variables(scope, suffix, ops.GraphKeys.LOCAL_VARIABLES)
 
 
+def get_trainable_variables(scope=None, suffix=None):
+  """Gets the list of trainable variables, filtered by scope and/or suffix.
+
+  Args:
+    scope: an optional scope for filtering the variables to return.
+    suffix: an optional suffix for filtering the variables to return.
+
+  Returns:
+    a list of variables in the trainable collection with scope and suffix.
+  """
+  return get_variables(scope, suffix, ops.GraphKeys.TRAINABLE_VARIABLES)
+
+
 def get_variables_to_restore(include=None, exclude=None):
   """Gets the list of the variables to restore.
 
@@ -410,7 +417,7 @@ def get_unique_variable(var_op_name):
   for candidate in candidates:
     if candidate.op.name == var_op_name:
       return candidate
-  raise ValueError('Variable %s does not uniquely identify a variable',
+  raise ValueError('Variable %s does not uniquely identify a variable' %
                    var_op_name)
 
 
@@ -438,7 +445,7 @@ def assign_from_values(var_names_to_values):
     var_value = var_names_to_values[var_name]
     var = ops.get_collection(ops.GraphKeys.GLOBAL_VARIABLES, var_name)
     if not var:
-      raise ValueError('Variable %s wasnt found', var_name)
+      raise ValueError('Variable %s wasn\'t found' % var_name)
     elif len(var) > 1:
       # tf.get_collection is just a filter on the prefix: find the exact match:
       found = False
@@ -449,7 +456,7 @@ def assign_from_values(var_names_to_values):
           break
 
       if not found:
-        raise ValueError('Variable %s doesnt uniquely identify a variable',
+        raise ValueError('Variable %s doesn\'t uniquely identify a variable' %
                          var_name)
     else:
       var = var[0]
@@ -491,68 +498,133 @@ def assign_from_values_fn(var_names_to_values):
   return callback
 
 
+# pylint: disable=protected-access
+# Currently variable_scope doesn't provide very good APIs to access
+# all variables under scope and retrieve and check existing scopes.
+def get_variable_full_name(var):
+  """Returns the full name of a variable.
+
+  For normal Variables, this is the same as the var.op.name.  For
+  sliced or PartitionedVariables, this name is the same for all the
+  slices/partitions. In both cases, this is normally the name used in
+  a checkpoint file.
+
+  Args:
+    var: A `Variable` object.
+
+  Returns:
+    A string that is the full name.
+  """
+  if var._save_slice_info:
+    return var._save_slice_info.full_name
+  else:
+    return var.op.name
+
+
 # TODO(nsilberman): add flag to load exponential moving averages instead
-def assign_from_checkpoint(model_path, var_list):
+#
+# TODO(sguada): Update docs in slim/g3doc/index.md to describe
+# the new feature where the var_list dictionary can have values that
+# are each a list of Variables.
+def assign_from_checkpoint(model_path, var_list, ignore_missing_vars=False):
   """Creates an operation to assign specific variables from a checkpoint.
 
   Args:
     model_path: The full path to the model checkpoint. To get latest checkpoint
         use `model_path = tf.train.latest_checkpoint(checkpoint_dir)`
-    var_list: A list of `Variable` objects or a dictionary mapping names in the
-        checkpoint to the corresponding variables to initialize. If empty or
-        None, it would return  no_op(), None.
+    var_list: A list of (possibly partitioned) `Variable` objects
+        or a dictionary mapping names in the checkpoint to the
+        corresponding variables or list of variables to initialize
+        from that checkpoint value. For partitioned Variables, the
+        name in the checkpoint must be the full variable, not the
+        name of the partitioned variable, eg. "my_var" rather than
+        "my_var/part_4". If empty, returns no_op(), {}.
+    ignore_missing_vars: Boolean, if True ignore variables missing in the
+        checkpoint with a warning instead of failing.
 
   Returns:
     the restore_op and the feed_dict that need to be run to restore var_list.
 
   Raises:
-    ValueError: If the checkpoint specified at `model_path` is missing one of
-      the variables in `var_list`.
+    ValueError: If `ignore_missing_vars` is False and the checkpoint specified
+        at `model_path` is missing one of the variables in `var_list`.
   """
-  reader = pywrap_tensorflow.NewCheckpointReader(model_path)
-
+  # Normalize var_list into a dictionary mapping names in the
+  # checkpoint to the list of variables to initialize from that
+  # checkpoint variable. Sliced (including partitioned) variables will
+  # end up under the same key.
+  grouped_vars = {}
   if isinstance(var_list, (tuple, list)):
-    var_list = {var.op.name: var for var in var_list}
+    for var in var_list:
+      ckpt_name = get_variable_full_name(var)
+      if ckpt_name not in grouped_vars:
+        grouped_vars[ckpt_name] = []
+      grouped_vars[ckpt_name].append(var)
 
+  else:
+    for ckpt_name, value in var_list.items():
+      if isinstance(value, (tuple, list)):
+        grouped_vars[ckpt_name] = value
+      else:
+        grouped_vars[ckpt_name] = [value]
+
+  # Read each checkpoint entry. Create a placeholder variable and
+  # add the (possibly sliced) data from the checkpoint to the feed_dict.
+  reader = pywrap_tensorflow.NewCheckpointReader(model_path)
   feed_dict = {}
   assign_ops = []
+  for ckpt_name in grouped_vars:
+    if not reader.has_tensor(ckpt_name):
+      log_str = 'Checkpoint is missing variable [%s]' % ckpt_name
+      if ignore_missing_vars:
+        logging.warning(log_str)
+        continue
+      else:
+        raise ValueError(log_str)
+    ckpt_value = reader.get_tensor(ckpt_name)
 
-  for checkpoint_var_name in var_list:
-    var = var_list[checkpoint_var_name]
-    if not reader.has_tensor(checkpoint_var_name):
-      raise ValueError(
-          'Checkpoint is missing variable [%s]' % checkpoint_var_name)
+    for var in grouped_vars[ckpt_name]:
+      placeholder_tensor = array_ops.placeholder(
+          dtype=var.dtype.base_dtype,
+          shape=var.get_shape(),
+          name='placeholder/' + var.op.name)
+      assign_ops.append(var.assign(placeholder_tensor))
 
-    var_value = reader.get_tensor(checkpoint_var_name)
-    placeholder_name = 'placeholder/' + var.op.name
-    placeholder_value = array_ops.placeholder(
-        dtype=var.dtype.base_dtype,
-        shape=var.get_shape(),
-        name=placeholder_name)
-    assign_ops.append(var.assign(placeholder_value))
+      if not var._save_slice_info:
+        if var.get_shape() != ckpt_value.shape:
+          raise ValueError(
+              'Total size of new array must be unchanged for %s '
+              'lh_shape: [%s], rh_shape: [%s]'
+              % (ckpt_name, str(ckpt_value.shape), str(var.get_shape())))
 
-    if var.get_shape() != var_value.shape:
-      raise ValueError(
-          'Total size of new array must be unchanged for %s '
-          'lh_shape: [%s], rh_shape: [%s]'
-          % (checkpoint_var_name, str(var_value.shape), str(var.get_shape())))
-
-    feed_dict[placeholder_value] = var_value.reshape(var.get_shape())
+        feed_dict[placeholder_tensor] = ckpt_value.reshape(ckpt_value.shape)
+      else:
+        slice_dims = zip(var._save_slice_info.var_offset,
+                         var._save_slice_info.var_shape)
+        slice_dims = [(start, start + size) for (start, size) in slice_dims]
+        slice_dims = [slice(*x) for x in slice_dims]
+        slice_value = ckpt_value[slice_dims]
+        slice_value = slice_value.reshape(var._save_slice_info.var_shape)
+        feed_dict[placeholder_tensor] = slice_value
 
   assign_op = control_flow_ops.group(*assign_ops)
   return assign_op, feed_dict
+# pylint: enable=protected-access
 
 
 def assign_from_checkpoint_fn(model_path, var_list, ignore_missing_vars=False,
                               reshape_variables=False):
   """Returns a function that assigns specific variables from a checkpoint.
 
+  If ignore_missing_vars is True and no variables are found in the checkpoint
+  it returns None.
+
   Args:
     model_path: The full path to the model checkpoint. To get latest checkpoint
         use `model_path = tf.train.latest_checkpoint(checkpoint_dir)`
     var_list: A list of `Variable` objects or a dictionary mapping names in the
-        checkpoint to the correspoing variables to initialize. If empty or None,
-        it would return  no_op(), None.
+        checkpoint to the corresponding variables to initialize. If empty or
+        `None`, it would return `no_op(), None`.
     ignore_missing_vars: Boolean, if True it would ignore variables missing in
         the checkpoint with a warning instead of failing.
     reshape_variables: Boolean, if True it would automatically reshape variables
@@ -561,12 +633,14 @@ def assign_from_checkpoint_fn(model_path, var_list, ignore_missing_vars=False,
 
   Returns:
     A function that takes a single argument, a `tf.Session`, that applies the
-    assignment operation.
+    assignment operation. If no matching variables were found in the checkpoint
+    then `None` is returned.
 
   Raises:
-    ValueError: If the checkpoint specified at `model_path` is missing one of
-      the variables in `var_list`.
+    ValueError: If var_list is empty.
   """
+  if not var_list:
+    raise ValueError('var_list cannot be empty')
   if ignore_missing_vars:
     reader = pywrap_tensorflow.NewCheckpointReader(model_path)
     if isinstance(var_list, dict):
@@ -581,10 +655,14 @@ def assign_from_checkpoint_fn(model_path, var_list, ignore_missing_vars=False,
         logging.warning(
             'Variable %s missing in checkpoint %s', var, model_path)
     var_list = available_vars
-  saver = tf_saver.Saver(var_list, reshape=reshape_variables)
-  def callback(session):
-    saver.restore(session, model_path)
-  return callback
+  if var_list:
+    saver = tf_saver.Saver(var_list, reshape=reshape_variables)
+    def callback(session):
+      saver.restore(session, model_path)
+    return callback
+  else:
+    logging.warning('No Variables to restore')
+    return None
 
 
 class VariableDeviceChooser(object):
