@@ -22,6 +22,7 @@ from __future__ import print_function
 
 import abc
 
+from tensorflow.python.eager import context
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
 from tensorflow.python.ops import array_ops
@@ -37,12 +38,14 @@ from tensorflow.python.util import nest
 
 def _get_variable_for(v):
   """Returns the ResourceVariable responsible for v, or v if not necessary."""
+  if context.in_eager_mode():
+    return v
   if v.op.type == "VarHandleOp":
     for var in variables.trainable_variables():
       if (isinstance(var, resource_variable_ops.ResourceVariable)
           and var.handle.op is v.op):
         return var
-    raise ValueError("Got %s but  could not locate source variable." % (str(v)))
+    raise ValueError("Got %s but could not locate source variable." % (str(v)))
   return v
 
 
@@ -66,6 +69,8 @@ def _deduplicate_indexed_slices(values, indices):
 
 
 def _var_key(var):
+  if context.in_eager_mode():
+    return var._shared_name  # pylint: disable=protected-access
   return (var.op.graph, var.op.name)
 
 
@@ -94,10 +99,18 @@ class _RefVariableProcessor(_OptimizableVariable):
 
   def update_op(self, optimizer, g):
     if isinstance(g, ops.Tensor):
-      return optimizer._apply_dense(g, self._v)  # pylint: disable=protected-access
+      update_op = optimizer._apply_dense(g, self._v)  # pylint: disable=protected-access
+      if self._v.constraint is not None:
+        with ops.control_dependencies([update_op]):
+          return self._v.assign(self._v.constraint(self._v))
+      else:
+        return update_op
     else:
       assert isinstance(g, ops.IndexedSlices), ("Gradient ", g, " is neither a "
                                                 "tensor nor IndexedSlices.")
+      if self._v.constraint is not None:
+        raise RuntimeError(
+            "Cannot use a constraint function on a sparse variable.")
       # pylint: disable=protected-access
       return optimizer._apply_sparse_duplicate_indices(g, self._v)
 
@@ -113,7 +126,12 @@ class _DenseReadResourceVariableProcessor(_OptimizableVariable):
 
   def update_op(self, optimizer, g):
     # pylint: disable=protected-access
-    return optimizer._resource_apply_dense(g, self._v.op.inputs[0])
+    update_op = optimizer._resource_apply_dense(g, self._v.op.inputs[0])
+    if self._v.constraint is not None:
+      with ops.control_dependencies([update_op]):
+        return self._v.assign(self._v.constraint(self._v))
+    else:
+      return update_op
 
 
 class _DenseResourceVariableProcessor(_OptimizableVariable):
@@ -128,9 +146,17 @@ class _DenseResourceVariableProcessor(_OptimizableVariable):
   def update_op(self, optimizer, g):
     # pylint: disable=protected-access
     if isinstance(g, ops.IndexedSlices):
+      if self._v.constraint is not None:
+        raise RuntimeError(
+            "Cannot use a constraint function on a sparse variable.")
       return optimizer._resource_apply_sparse_duplicate_indices(
           g.values, self._v, g.indices)
-    return optimizer._resource_apply_dense(g, self._v)
+    update_op = optimizer._resource_apply_dense(g, self._v)
+    if self._v.constraint is not None:
+      with ops.control_dependencies([update_op]):
+        return self._v.assign(self._v.constraint(self._v))
+    else:
+      return update_op
 
 
 class _StreamingModelPortProcessor(_OptimizableVariable):
@@ -148,6 +174,8 @@ class _StreamingModelPortProcessor(_OptimizableVariable):
 
 def _get_processor(v):
   """The processor of v."""
+  if context.in_eager_mode():
+    return _DenseResourceVariableProcessor(v)
   if v.op.type == "VarHandleOp":
     return _DenseResourceVariableProcessor(v)
   if isinstance(v, variables.Variable):
@@ -452,7 +480,9 @@ class Optimizer(object):
           continue
         # We colocate all ops created in _apply_dense or _apply_sparse
         # on the same device as the variable.
-        with ops.name_scope("update_" + var.op.name), ops.colocate_with(var):
+        # TODO(apassos): figure out how to get the variable name here.
+        scope_name = var.op.name if context.in_graph_mode() else ""
+        with ops.name_scope("update_" + scope_name), ops.colocate_with(var):
           update_ops.append(processor.update_op(self, grad))
       if global_step is None:
         apply_updates = self._finish(update_ops, name)
@@ -554,7 +584,7 @@ class Optimizer(object):
       grad: A `Tensor`.
       var: A `Variable` object.
 
-    Return:
+    Returns:
       An `Operation`.
     """
     raise NotImplementedError()
@@ -667,7 +697,7 @@ class Optimizer(object):
       grad: `IndexedSlices`, with no repeated indices.
       var: A `Variable` object.
 
-    Return:
+    Returns:
       An `Operation`.
     """
     raise NotImplementedError()
@@ -717,7 +747,7 @@ class Optimizer(object):
       val: A `Tensor`.  The initial value of the slot.
       slot_name: Name for the slot.
       op_name: Name to use when scoping the Variable that
-        needs to be created for  the slot.
+        needs to be created for the slot.
 
     Returns:
       A `Variable` object.
@@ -738,7 +768,7 @@ class Optimizer(object):
       dtype: Type of the value of the slot.
       slot_name: Name for the slot.
       op_name: Name to use when scoping the Variable that
-        needs to be created for  the slot.
+        needs to be created for the slot.
 
     Returns:
       A `Variable` object.
@@ -756,7 +786,7 @@ class Optimizer(object):
       var: A `Variable` object.
       slot_name: Name for the slot.
       op_name: Name to use when scoping the Variable that
-        needs to be created for  the slot.
+        needs to be created for the slot.
 
     Returns:
       A `Variable` object.
